from dataclasses import dataclass
from enum import Enum, IntEnum
from typing import Optional, Dict, List


@dataclass
class ProviderConfig:
    base_uri: str
    default_headers: Optional[Dict[str, str]] = None


class ResponseValidationMode(IntEnum):
    LAX = 0
    STRICT = 1


class LineType(str, Enum):
    HEADERS = 'header'
    DATA = 'data'


class Dataset:
    keys: str
    data: List[str]

    def __init__(self, keys: str):
        self.keys = keys
        self.data = []


class ParseTarget:
    to_key: str
    as_list: bool
    to_root: bool

    def __init__(self, to_key: str = '', as_list: bool = False, to_root: bool = False):
        self.to_key = to_key
        self.as_list = as_list
        self.to_root = to_root



from typing import Dict, Union

from aspxstats.validation import AttributeSchema

SEARCHFORPLAYERS_RESPONSE_SCHEMA: Dict[str, Union[dict, AttributeSchema]] = {
    'asof': AttributeSchema(type=str, is_numeric=True),
    'results': AttributeSchema(type=list, children={
        'n': AttributeSchema(type=str, is_numeric=True),
        'pid': AttributeSchema(type=str, is_numeric=True),
        'nick': AttributeSchema(type=str),
        'score': AttributeSchema(type=str, is_numeric=True)
    })
}

GETLEADERBOARD_RESPONSE_SCHEMA: Dict[str, Union[dict, AttributeSchema]] = {
    'size': AttributeSchema(type=str, is_numeric=True),
    'asof': AttributeSchema(type=str, is_numeric=True),
    'entries': AttributeSchema(type=list, children={
        'n': AttributeSchema(type=str, is_numeric=True),
        'pid': AttributeSchema(type=str, is_numeric=True),
        'nick': AttributeSchema(type=str),
        'playerrank': AttributeSchema(type=str, is_numeric=True),
        'countrycode': AttributeSchema(type=str)
    })
}

GETPLAYERINFO_GENERAL_STATS_RESPONSE_SCHEMA: Dict[str, Union[dict, AttributeSchema]] = {
    'asof': AttributeSchema(type=str, is_numeric=True),
    'player': {
        'pid': AttributeSchema(type=str, is_numeric=True),
        'nick': AttributeSchema(type=str),
        'scor': AttributeSchema(type=str, is_numeric=True),
        'jond': AttributeSchema(type=str, is_numeric=True),
        'wins': AttributeSchema(type=str, is_numeric=True),
        'loss': AttributeSchema(type=str, is_numeric=True),
        'mode0': AttributeSchema(type=str, is_numeric=True),
        'mode1': AttributeSchema(type=str, is_numeric=True),
        'mode2': AttributeSchema(type=str, is_numeric=True),
        'time': AttributeSchema(type=str, is_numeric=True),
        'smoc': AttributeSchema(type=str, is_numeric=True),
        'cmsc': AttributeSchema(type=str, is_numeric=True),
        'osaa': AttributeSchema(type=str, is_floaty=True),
        'kill': AttributeSchema(type=str, is_numeric=True),
        'kila': AttributeSchema(type=str, is_numeric=True),
        'deth': AttributeSchema(type=str, is_numeric=True),
        'suic': AttributeSchema(type=str, is_numeric=True),
        'bksk': AttributeSchema(type=str, is_numeric=True),
        'wdsk': AttributeSchema(type=str, is_numeric=True),
        'tvcr': AttributeSchema(type=str, is_numeric=True),
        'topr': AttributeSchema(type=str, is_numeric=True),
        'klpm': AttributeSchema(type=str, is_floaty=True),
        'dtpm': AttributeSchema(type=str, is_floaty=True),
        'ospm': AttributeSchema(type=str, is_floaty=True),
        'klpr': AttributeSchema(type=str, is_floaty=True),
        'dtpr': AttributeSchema(type=str, is_floaty=True),
        'twsc': AttributeSchema(type=str, is_numeric=True),
        'cpcp': AttributeSchema(type=str, is_numeric=True),
        'cacp': AttributeSchema(type=str, is_numeric=True),
        'dfcp': AttributeSchema(type=str, is_numeric=True),
        'heal': AttributeSchema(type=str, is_numeric=True),
        'rviv': AttributeSchema(type=str, is_numeric=True),
        'rsup': AttributeSchema(type=str, is_numeric=True),
        'rpar': AttributeSchema(type=str, is_numeric=True),
        'tgte': AttributeSchema(type=str, is_numeric=True),
        'dkas': AttributeSchema(type=str, is_numeric=True),
        'dsab': AttributeSchema(type=str, is_numeric=True),
        'cdsc': AttributeSchema(type=str, is_numeric=True),
        'rank': AttributeSchema(type=str, is_numeric=True),
        'kick': AttributeSchema(type=str, is_numeric=True),
        'bbrs': AttributeSchema(type=str, is_numeric=True),
        'tcdr': AttributeSchema(type=str, is_numeric=True),
        'ban': AttributeSchema(type=str, is_numeric=True),
        'lbtl': AttributeSchema(type=str, is_numeric=True),
        'vrk': AttributeSchema(type=str, is_numeric=True),
        'tsql': AttributeSchema(type=str, is_numeric=True),
        'tsqm': AttributeSchema(type=str, is_numeric=True),
        'tlwf': AttributeSchema(type=str, is_numeric=True),
        'mvks': AttributeSchema(type=str, is_numeric=True),
        'vmks': AttributeSchema(type=str, is_numeric=True),
        'mvns': AttributeSchema(type=str),
        'mvrs': AttributeSchema(type=str, is_numeric=True),
        'vmns': AttributeSchema(type=str),
        'vmrs': AttributeSchema(type=str, is_numeric=True),
        'fkit': AttributeSchema(type=str, is_numeric=True),
        'fmap': AttributeSchema(type=str, is_numeric=True),
        'fveh': AttributeSchema(type=str, is_numeric=True),
        'fwea': AttributeSchema(type=str, is_numeric=True),
        'tnv': AttributeSchema(type=str, is_numeric=True),
        'tgm': AttributeSchema(type=str, is_numeric=True),
        'wtm-0': AttributeSchema(type=str, is_numeric=True),
        'wtm-1': AttributeSchema(type=str, is_numeric=True),
        'wtm-2': AttributeSchema(type=str, is_numeric=True),
        'wtm-3': AttributeSchema(type=str, is_numeric=True),
        'wtm-4': AttributeSchema(type=str, is_numeric=True),
        'wtm-5': AttributeSchema(type=str, is_numeric=True),
        'wtm-6': AttributeSchema(type=str, is_numeric=True),
        'wtm-7': AttributeSchema(type=str, is_numeric=True),
        'wtm-8': AttributeSchema(type=str, is_numeric=True),
        'wtm-9': AttributeSchema(type=str, is_numeric=True),
        'wtm-10': AttributeSchema(type=str, is_numeric=True),
        'wtm-11': AttributeSchema(type=str, is_numeric=True),
        'wtm-12': AttributeSchema(type=str, is_numeric=True),
        'wtm-13': AttributeSchema(type=str, is_numeric=True),
        'wkl-0': AttributeSchema(type=str, is_numeric=True),
        'wkl-1': AttributeSchema(type=str, is_numeric=True),
        'wkl-2': AttributeSchema(type=str, is_numeric=True),
        'wkl-3': AttributeSchema(type=str, is_numeric=True),
        'wkl-4': AttributeSchema(type=str, is_numeric=True),
        'wkl-5': AttributeSchema(type=str, is_numeric=True),
        'wkl-6': AttributeSchema(type=str, is_numeric=True),
        'wkl-7': AttributeSchema(type=str, is_numeric=True),
        'wkl-8': AttributeSchema(type=str, is_numeric=True),
        'wkl-9': AttributeSchema(type=str, is_numeric=True),
        'wkl-10': AttributeSchema(type=str, is_numeric=True),
        'wkl-11': AttributeSchema(type=str, is_numeric=True),
        'wkl-12': AttributeSchema(type=str, is_numeric=True),
        'wkl-13': AttributeSchema(type=str, is_numeric=True),
        'wdt-0': AttributeSchema(type=str, is_numeric=True),
        'wdt-1': AttributeSchema(type=str, is_numeric=True),
        'wdt-2': AttributeSchema(type=str, is_numeric=True),
        'wdt-3': AttributeSchema(type=str, is_numeric=True),
        'wdt-4': AttributeSchema(type=str, is_numeric=True),
        'wdt-5': AttributeSchema(type=str, is_numeric=True),
        'wdt-6': AttributeSchema(type=str, is_numeric=True),
        'wdt-7': AttributeSchema(type=str, is_numeric=True),
        'wdt-8': AttributeSchema(type=str, is_numeric=True),
        'wdt-9': AttributeSchema(type=str, is_numeric=True),
        'wdt-10': AttributeSchema(type=str, is_numeric=True),
        'wdt-11': AttributeSchema(type=str, is_numeric=True),
        'wdt-12': AttributeSchema(type=str, is_numeric=True),
        'wdt-13': AttributeSchema(type=str, is_numeric=True),
        'wac-0': AttributeSchema(type=str, is_floaty=True),
        'wac-1': AttributeSchema(type=str, is_floaty=True),
        'wac-2': AttributeSchema(type=str, is_floaty=True),
        'wac-3': AttributeSchema(type=str, is_floaty=True),
        'wac-4': AttributeSchema(type=str, is_floaty=True),
        'wac-5': AttributeSchema(type=str, is_floaty=True),
        'wac-6': AttributeSchema(type=str, is_floaty=True),
        'wac-7': AttributeSchema(type=str, is_floaty=True),
        'wac-8': AttributeSchema(type=str, is_floaty=True),
        'wac-9': AttributeSchema(type=str, is_floaty=True),
        'wac-10': AttributeSchema(type=str, is_floaty=True),
        'wac-11': AttributeSchema(type=str, is_floaty=True),
        'wac-12': AttributeSchema(type=str, is_floaty=True),
        'wac-13': AttributeSchema(type=str, is_floaty=True),
        'wkd-0': AttributeSchema(type=str, is_ratio=True),
        'wkd-1': AttributeSchema(type=str, is_ratio=True),
        'wkd-2': AttributeSchema(type=str, is_ratio=True),
        'wkd-3': AttributeSchema(type=str, is_ratio=True),
        'wkd-4': AttributeSchema(type=str, is_ratio=True),
        'wkd-5': AttributeSchema(type=str, is_ratio=True),
        'wkd-6': AttributeSchema(type=str, is_ratio=True),
        'wkd-7': AttributeSchema(type=str, is_ratio=True),
        'wkd-8': AttributeSchema(type=str, is_ratio=True),
        'wkd-9': AttributeSchema(type=str, is_ratio=True),
        'wkd-10': AttributeSchema(type=str, is_ratio=True),
        'wkd-11': AttributeSchema(type=str, is_ratio=True),
        'wkd-12': AttributeSchema(type=str, is_ratio=True),
        'wkd-13': AttributeSchema(type=str, is_ratio=True),
        'vtm-0': AttributeSchema(type=str, is_numeric=True),
        'vtm-1': AttributeSchema(type=str, is_numeric=True),
        'vtm-2': AttributeSchema(type=str, is_numeric=True),
        'vtm-3': AttributeSchema(type=str, is_numeric=True),
        'vtm-4': AttributeSchema(type=str, is_numeric=True),
        'vtm-5': AttributeSchema(type=str, is_numeric=True),
        'vtm-6': AttributeSchema(type=str, is_numeric=True),
        'vkl-0': AttributeSchema(type=str, is_numeric=True),
        'vkl-1': AttributeSchema(type=str, is_numeric=True),
        'vkl-2': AttributeSchema(type=str, is_numeric=True),
        'vkl-3': AttributeSchema(type=str, is_numeric=True),
        'vkl-4': AttributeSchema(type=str, is_numeric=True),
        'vkl-5': AttributeSchema(type=str, is_numeric=True),
        'vkl-6': AttributeSchema(type=str, is_numeric=True),
        'vdt-0': AttributeSchema(type=str, is_numeric=True),
        'vdt-1': AttributeSchema(type=str, is_numeric=True),
        'vdt-2': AttributeSchema(type=str, is_numeric=True),
        'vdt-3': AttributeSchema(type=str, is_numeric=True),
        'vdt-4': AttributeSchema(type=str, is_numeric=True),
        'vdt-5': AttributeSchema(type=str, is_numeric=True),
        'vdt-6': AttributeSchema(type=str, is_numeric=True),
        'vkd-0': AttributeSchema(type=str, is_ratio=True),
        'vkd-1': AttributeSchema(type=str, is_ratio=True),
        'vkd-2': AttributeSchema(type=str, is_ratio=True),
        'vkd-3': AttributeSchema(type=str, is_ratio=True),
        'vkd-4': AttributeSchema(type=str, is_ratio=True),
        'vkd-5': AttributeSchema(type=str, is_ratio=True),
        'vkd-6': AttributeSchema(type=str, is_ratio=True),
        'vkr-0': AttributeSchema(type=str, is_numeric=True),
        'vkr-1': AttributeSchema(type=str, is_numeric=True),
        'vkr-2': AttributeSchema(type=str, is_numeric=True),
        'vkr-3': AttributeSchema(type=str, is_numeric=True),
        'vkr-4': AttributeSchema(type=str, is_numeric=True),
        'vkr-5': AttributeSchema(type=str, is_numeric=True),
        'vkr-6': AttributeSchema(type=str, is_numeric=True),
        'atm-0': AttributeSchema(type=str, is_numeric=True),
        'atm-1': AttributeSchema(type=str, is_numeric=True),
        'atm-2': AttributeSchema(type=str, is_numeric=True),
        'atm-3': AttributeSchema(type=str, is_numeric=True),
        'atm-4': AttributeSchema(type=str, is_numeric=True),
        'atm-5': AttributeSchema(type=str, is_numeric=True),
        'atm-6': AttributeSchema(type=str, is_numeric=True),
        'atm-7': AttributeSchema(type=str, is_numeric=True),
        'atm-8': AttributeSchema(type=str, is_numeric=True),
        'atm-9': AttributeSchema(type=str, is_numeric=True),
        'awn-0': AttributeSchema(type=str, is_numeric=True),
        'awn-1': AttributeSchema(type=str, is_numeric=True),
        'awn-2': AttributeSchema(type=str, is_numeric=True),
        'awn-3': AttributeSchema(type=str, is_numeric=True),
        'awn-4': AttributeSchema(type=str, is_numeric=True),
        'awn-5': AttributeSchema(type=str, is_numeric=True),
        'awn-6': AttributeSchema(type=str, is_numeric=True),
        'awn-7': AttributeSchema(type=str, is_numeric=True),
        'awn-8': AttributeSchema(type=str, is_numeric=True),
        'awn-9': AttributeSchema(type=str, is_numeric=True),
        'alo-0': AttributeSchema(type=str, is_numeric=True),
        'alo-1': AttributeSchema(type=str, is_numeric=True),
        'alo-2': AttributeSchema(type=str, is_numeric=True),
        'alo-3': AttributeSchema(type=str, is_numeric=True),
        'alo-4': AttributeSchema(type=str, is_numeric=True),
        'alo-5': AttributeSchema(type=str, is_numeric=True),
        'alo-6': AttributeSchema(type=str, is_numeric=True),
        'alo-7': AttributeSchema(type=str, is_numeric=True),
        'alo-8': AttributeSchema(type=str, is_numeric=True),
        'alo-9': AttributeSchema(type=str, is_numeric=True),
        'abr-0': AttributeSchema(type=str, is_numeric=True),
        'abr-1': AttributeSchema(type=str, is_numeric=True),
        'abr-2': AttributeSchema(type=str, is_numeric=True),
        'abr-3': AttributeSchema(type=str, is_numeric=True),
        'abr-4': AttributeSchema(type=str, is_numeric=True),
        'abr-5': AttributeSchema(type=str, is_numeric=True),
        'abr-6': AttributeSchema(type=str, is_numeric=True),
        'abr-7': AttributeSchema(type=str, is_numeric=True),
        'abr-8': AttributeSchema(type=str, is_numeric=True),
        'abr-9': AttributeSchema(type=str, is_numeric=True),
        'ktm-0': AttributeSchema(type=str, is_numeric=True),
        'ktm-1': AttributeSchema(type=str, is_numeric=True),
        'ktm-2': AttributeSchema(type=str, is_numeric=True),
        'ktm-3': AttributeSchema(type=str, is_numeric=True),
        'ktm-4': AttributeSchema(type=str, is_numeric=True),
        'ktm-5': AttributeSchema(type=str, is_numeric=True),
        'ktm-6': AttributeSchema(type=str, is_numeric=True),
        'kkl-0': AttributeSchema(type=str, is_numeric=True),
        'kkl-1': AttributeSchema(type=str, is_numeric=True),
        'kkl-2': AttributeSchema(type=str, is_numeric=True),
        'kkl-3': AttributeSchema(type=str, is_numeric=True),
        'kkl-4': AttributeSchema(type=str, is_numeric=True),
        'kkl-5': AttributeSchema(type=str, is_numeric=True),
        'kkl-6': AttributeSchema(type=str, is_numeric=True),
        'kdt-0': AttributeSchema(type=str, is_numeric=True),
        'kdt-1': AttributeSchema(type=str, is_numeric=True),
        'kdt-2': AttributeSchema(type=str, is_numeric=True),
        'kdt-3': AttributeSchema(type=str, is_numeric=True),
        'kdt-4': AttributeSchema(type=str, is_numeric=True),
        'kdt-5': AttributeSchema(type=str, is_numeric=True),
        'kdt-6': AttributeSchema(type=str, is_numeric=True),
        'kkd-0': AttributeSchema(type=str, is_ratio=True),
        'kkd-1': AttributeSchema(type=str, is_ratio=True),
        'kkd-2': AttributeSchema(type=str, is_ratio=True),
        'kkd-3': AttributeSchema(type=str, is_ratio=True),
        'kkd-4': AttributeSchema(type=str, is_ratio=True),
        'kkd-5': AttributeSchema(type=str, is_ratio=True),
        'kkd-6': AttributeSchema(type=str, is_ratio=True),
        'de-6': AttributeSchema(type=str, is_numeric=True),
        'de-7': AttributeSchema(type=str, is_numeric=True),
        'de-8': AttributeSchema(type=str, is_numeric=True),
    }
}

GETPLAYERINFO_MAP_STATS_RESPONSE_SCHEMA: Dict[str, Union[dict, AttributeSchema]] = {
    'asof': AttributeSchema(type=str, is_numeric=True),
    'player': {
        'pid': AttributeSchema(type=str, is_numeric=True),
        'nick': AttributeSchema(type=str),
        'mtm-0': AttributeSchema(type=str, is_numeric=True),
        'mtm-1': AttributeSchema(type=str, is_numeric=True),
        'mtm-2': AttributeSchema(type=str, is_numeric=True),
        'mtm-3': AttributeSchema(type=str, is_numeric=True),
        'mtm-4': AttributeSchema(type=str, is_numeric=True),
        'mtm-5': AttributeSchema(type=str, is_numeric=True),
        'mtm-6': AttributeSchema(type=str, is_numeric=True),
        'mtm-10': AttributeSchema(type=str, is_numeric=True),
        'mtm-11': AttributeSchema(type=str, is_numeric=True),
        'mtm-12': AttributeSchema(type=str, is_numeric=True),
        'mtm-100': AttributeSchema(type=str, is_numeric=True),
        'mtm-101': AttributeSchema(type=str, is_numeric=True),
        'mtm-102': AttributeSchema(type=str, is_numeric=True),
        'mtm-103': AttributeSchema(type=str, is_numeric=True),
        'mtm-104': AttributeSchema(type=str, is_numeric=True),
        'mtm-105': AttributeSchema(type=str, is_numeric=True),
        'mtm-110': AttributeSchema(type=str, is_numeric=True),
        'mtm-200': AttributeSchema(type=str, is_numeric=True),
        'mtm-201': AttributeSchema(type=str, is_numeric=True),
        'mtm-202': AttributeSchema(type=str, is_numeric=True),
        'mtm-300': AttributeSchema(type=str, is_numeric=True),
        'mtm-301': AttributeSchema(type=str, is_numeric=True),
        'mtm-302': AttributeSchema(type=str, is_numeric=True),
        'mtm-303': AttributeSchema(type=str, is_numeric=True),
        'mtm-304': AttributeSchema(type=str, is_numeric=True),
        'mtm-305': AttributeSchema(type=str, is_numeric=True),
        'mtm-306': AttributeSchema(type=str, is_numeric=True),
        'mtm-307': AttributeSchema(type=str, is_numeric=True),
        'mtm-601': AttributeSchema(type=str, is_numeric=True),
        'mwn-0': AttributeSchema(type=str, is_numeric=True),
        'mwn-1': AttributeSchema(type=str, is_numeric=True),
        'mwn-2': AttributeSchema(type=str, is_numeric=True),
        'mwn-3': AttributeSchema(type=str, is_numeric=True),
        'mwn-4': AttributeSchema(type=str, is_numeric=True),
        'mwn-5': AttributeSchema(type=str, is_numeric=True),
        'mwn-6': AttributeSchema(type=str, is_numeric=True),
        'mwn-10': AttributeSchema(type=str, is_numeric=True),
        'mwn-11': AttributeSchema(type=str, is_numeric=True),
        'mwn-12': AttributeSchema(type=str, is_numeric=True),
        'mwn-100': AttributeSchema(type=str, is_numeric=True),
        'mwn-101': AttributeSchema(type=str, is_numeric=True),
        'mwn-102': AttributeSchema(type=str, is_numeric=True),
        'mwn-103': AttributeSchema(type=str, is_numeric=True),
        'mwn-104': AttributeSchema(type=str, is_numeric=True),
        'mwn-105': AttributeSchema(type=str, is_numeric=True),
        'mwn-110': AttributeSchema(type=str, is_numeric=True),
        'mwn-200': AttributeSchema(type=str, is_numeric=True),
        'mwn-201': AttributeSchema(type=str, is_numeric=True),
        'mwn-202': AttributeSchema(type=str, is_numeric=True),
        'mwn-300': AttributeSchema(type=str, is_numeric=True),
        'mwn-301': AttributeSchema(type=str, is_numeric=True),
        'mwn-302': AttributeSchema(type=str, is_numeric=True),
        'mwn-303': AttributeSchema(type=str, is_numeric=True),
        'mwn-304': AttributeSchema(type=str, is_numeric=True),
        'mwn-305': AttributeSchema(type=str, is_numeric=True),
        'mwn-306': AttributeSchema(type=str, is_numeric=True),
        'mwn-307': AttributeSchema(type=str, is_numeric=True),
        'mwn-601': AttributeSchema(type=str, is_numeric=True),
        'mls-0': AttributeSchema(type=str, is_numeric=True),
        'mls-1': AttributeSchema(type=str, is_numeric=True),
        'mls-2': AttributeSchema(type=str, is_numeric=True),
        'mls-3': AttributeSchema(type=str, is_numeric=True),
        'mls-4': AttributeSchema(type=str, is_numeric=True),
        'mls-5': AttributeSchema(type=str, is_numeric=True),
        'mls-6': AttributeSchema(type=str, is_numeric=True),
        'mls-10': AttributeSchema(type=str, is_numeric=True),
        'mls-11': AttributeSchema(type=str, is_numeric=True),
        'mls-12': AttributeSchema(type=str, is_numeric=True),
        'mls-100': AttributeSchema(type=str, is_numeric=True),
        'mls-101': AttributeSchema(type=str, is_numeric=True),
        'mls-102': AttributeSchema(type=str, is_numeric=True),
        'mls-103': AttributeSchema(type=str, is_numeric=True),
        'mls-104': AttributeSchema(type=str, is_numeric=True),
        'mls-105': AttributeSchema(type=str, is_numeric=True),
        'mls-110': AttributeSchema(type=str, is_numeric=True),
        'mls-200': AttributeSchema(type=str, is_numeric=True),
        'mls-201': AttributeSchema(type=str, is_numeric=True),
        'mls-202': AttributeSchema(type=str, is_numeric=True),
        'mls-300': AttributeSchema(type=str, is_numeric=True),
        'mls-301': AttributeSchema(type=str, is_numeric=True),
        'mls-302': AttributeSchema(type=str, is_numeric=True),
        'mls-303': AttributeSchema(type=str, is_numeric=True),
        'mls-304': AttributeSchema(type=str, is_numeric=True),
        'mls-305': AttributeSchema(type=str, is_numeric=True),
        'mls-306': AttributeSchema(type=str, is_numeric=True),
        'mls-307': AttributeSchema(type=str, is_numeric=True),
        'mls-601': AttributeSchema(type=str, is_numeric=True)
    }
}



from typing import Union, Optional

from .client import AspxClient
from .types import SearchMatchType, SearchSortOrder, PlayerSearchResponse, StatsProvider, LeaderboardType, \
    ScoreLeaderboardId, WeaponLeaderboardId, VehicleLeaderboardId, KitLeaderboardId, LeaderboardResponse, \
    PlayerinfoKeySet
from ..types import ResponseValidationMode


def searchforplayers(
        nick: str,
        where: SearchMatchType = SearchMatchType.EQUALS,
        sort: SearchSortOrder = SearchSortOrder.ASCENDING,
        provider: StatsProvider = StatsProvider.BF2HUB,
        timeout: float = 2.0,
        response_validation_mode: ResponseValidationMode = ResponseValidationMode.LAX
) -> PlayerSearchResponse:
    with AspxClient(provider, timeout, response_validation_mode) as client:
        return client.searchforplayers(nick, where, sort)


def searchforplayers_raw(
        nick: str,
        where: SearchMatchType = SearchMatchType.EQUALS,
        sort: SearchSortOrder = SearchSortOrder.ASCENDING,
        provider: StatsProvider = StatsProvider.BF2HUB,
        timeout: float = 2.0,
        response_validation_mode: ResponseValidationMode = ResponseValidationMode.LAX
) -> dict:
    with AspxClient(provider, timeout, response_validation_mode) as client:
        return client.searchforplayers_raw(nick, where, sort)


def getleaderboard(
        leaderboard_type: LeaderboardType = LeaderboardType.SCORE,
        leaderboard_id: Union[
            ScoreLeaderboardId,
            WeaponLeaderboardId,
            VehicleLeaderboardId,
            KitLeaderboardId
        ] = ScoreLeaderboardId.OVERALL,
        pos: int = 1,
        before: int = 0,
        after: int = 19,
        pid: Optional[int] = None,
        provider: StatsProvider = StatsProvider.BF2HUB,
        timeout: float = 2.0,
        response_validation_mode: ResponseValidationMode = ResponseValidationMode.LAX
) -> LeaderboardResponse:
    with AspxClient(provider, timeout, response_validation_mode) as client:
        return client.getleaderboard(leaderboard_type, leaderboard_id, pos, before, after, pid)


def getleaderboard_raw(
        leaderboard_type: LeaderboardType = LeaderboardType.SCORE,
        leaderboard_id: Union[
            ScoreLeaderboardId,
            WeaponLeaderboardId,
            VehicleLeaderboardId,
            KitLeaderboardId
        ] = ScoreLeaderboardId.OVERALL,
        pos: int = 1,
        before: int = 0,
        after: int = 19,
        pid: Optional[int] = None,
        provider: StatsProvider = StatsProvider.BF2HUB,
        timeout: float = 2.0,
        response_validation_mode: ResponseValidationMode = ResponseValidationMode.LAX
) -> dict:
    with AspxClient(provider, timeout, response_validation_mode) as client:
        return client.getleaderboard_raw(leaderboard_type, leaderboard_id, pos, before, after, pid)


def getplayerinfo_raw(
        pid: int,
        key_set: PlayerinfoKeySet = PlayerinfoKeySet.GENERAL_STATS,
        provider: StatsProvider = StatsProvider.BF2HUB,
        timeout: float = 2.0,
        response_validation_mode: ResponseValidationMode = ResponseValidationMode.LAX
) -> dict:
    with AspxClient(provider, timeout, response_validation_mode) as client:
        return client.getplayerinfo_raw(pid, key_set)



import re
from typing import Dict, Optional, Tuple, List
from urllib.parse import urljoin

import requests as requests

from .exceptions import ClientError, InvalidResponseError, Error
from .types import LineType, Dataset, ParseTarget, ResponseValidationMode


class AspxClient:
    base_uri: str
    default_headers: Dict[str, str]
    timeout: float
    response_validation_mode: ResponseValidationMode

    session: requests.Session
    not_found_regex: re.Pattern

    def __init__(
            self,
            base_uri: str,
            default_headers: Dict[str, str],
            timeout: float,
            response_validation_mode: ResponseValidationMode
    ):
        self.base_uri = base_uri
        self.default_headers = default_headers
        self.timeout = timeout
        self.response_validation_mode = response_validation_mode

        self.session = requests.session()
        self.session.headers = default_headers

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.close()

    def close(self) -> None:
        self.session.close()

    def get_aspx_data(self, endpoint: str, params: Optional[Dict[str, str]] = None) -> str:
        """
        Fetch raw, unparsed data from a .aspx endpoint
        :param endpoint: (relative) URL of the endpoint
        :param params: query params to send as part of the request
        :return: raw aspx data as a string
        """
        url = urljoin(self.base_uri, endpoint)
        try:
            response = self.session.get(url, params=params, timeout=self.timeout)

            if response.ok:
                return response.text
            else:
                raise ClientError(f'Failed to fetch ASPX data (HTTP/{response.status_code})')
        except requests.RequestException as e:
            raise ClientError(f'Failed to fetch ASPX data: {e}')

    @staticmethod
    def is_valid_aspx_response(
            raw_data: str,
            validation_mode: ResponseValidationMode = ResponseValidationMode.STRICT
    ) -> Tuple[bool, bool]:
        lines = raw_data.split('\n')
        first_line, last_line = lines[0], lines[-1]

        """
        Last line contains an indicator for the message length (excluding delimiters and excluding the last line)
        $	133	$
        => validate message length matches indicated length
        """
        actual_length = AspxClient.determine_actual_response_length(lines)
        indicated_length = AspxClient.get_indicated_response_length(last_line)

        response_valid = (first_line.strip() == 'O' and
                          (actual_length == indicated_length or validation_mode is ResponseValidationMode.LAX)
                          and AspxClient.are_response_datasets_valid(raw_data))

        """
        Each project handles player not found errors a little different
        BF2Hub returns "E\t998" in the first line (all endpoints)
        PlayBF2 returns a converged list of headers and dummy values in the first line (getplayerinfo)
        PlayBF2 returns a line containing "player [...] not found" (getmapinfo [unofficial])
        """
        not_found_error = not response_valid and (
                first_line == 'E\t998'
                or first_line.startswith('O	H	asof	D')
                or next((True for line in lines if 'player' in line.casefold() and 'not found' in line.casefold()), False)
        )

        return response_valid, not_found_error

    @staticmethod
    def determine_actual_response_length(lines: List[str]) -> int:
        """
        Determine the actual response length by counting the non-delimited characters
        :param lines: complete aspx response split into lines
        :return: actual length of the response
        """
        # Ignore the last line, since it should contain the length indicator
        return sum(len(line.replace('\t', '')) for line in lines[:-1])

    @staticmethod
    def get_indicated_response_length(indicator_line: str) -> int:
        """
        Get the indicated response length from the indicator line (last line) of the aspx response
        :param indicator_line: line containing response length indicator "$	14	$" (last line of response)
        :return: indicated length of the response (-1 in case the line does not contain a valid/parseable indicator)
        """
        if not indicator_line.startswith('$\t') or not indicator_line.endswith('\t$'):
            return -1

        try:
            return int(indicator_line.strip('\t$'))
        except ValueError:
            return -1

    @staticmethod
    def are_response_datasets_valid(raw_data: str) -> bool:
        datasets = AspxClient.extract_datasets_from_response(raw_data)
        return all(AspxClient.is_valid_dataset(dataset) for dataset in datasets)

    @staticmethod
    def is_valid_dataset(dataset: Dataset) -> bool:
        """
        Ensure that all value lines of a dataset have the same number of columns as the key line
        :param dataset: dataset to test
        :return: True, if column numbers match, else False
        """
        key_columns = dataset.keys.count('\t')
        return all(value_line.count('\t') == key_columns for value_line in dataset.data)

    @staticmethod
    def parse_aspx_response(raw_data: str, targets: List[ParseTarget]) -> dict:
        """
        Parse raw aspx data into a dictionary
        :param raw_data: raw aspx data as a string
        :param targets: targets to parse data into (defines how datasets in response are added to dictionary structure)
        :return: aspx data as dictionary (structure varies based on endpoint/targets parameter)
        """
        datasets = AspxClient.extract_datasets_from_response(raw_data)

        """
        For most endpoints we get two datasets: one containing metadata and one containing results.
        A common breach of this design: PlayBF2 does not support the rising star leaderboard. If you request it anyway,
        you get a single, empty dataset (which we cannot sensibly parse into the usual leaderboard response format):
        O
        H    size    asof
        $    10    $
        => raise an error if we have fewer dataset than targets to parse to
        """
        if len(datasets) < len(targets):
            raise InvalidResponseError('Received invalid number of datasets from list endpoint')

        return AspxClient.build_dict_from_datasets(datasets, targets)

    @staticmethod
    def extract_datasets_from_response(raw_data: str) -> List[Dataset]:
        lines = raw_data.split('\n')
        datasets: List[Dataset] = []
        # We should see a header line first (since we skip the "status" line)
        current_line_type: LineType = LineType.HEADERS
        data_line_index = 0
        for line in lines[1:]:
            if line[:2] == 'H\t':
                # Line starts with header marker => create and append new dataset
                current_line_type = LineType.HEADERS
                datasets.append(Dataset(line[2:]))
            elif line[:2] == 'D\t':
                # Line starts with data marker => add data line to current dataset
                if current_line_type == LineType.DATA:
                    # Data row is followed by another data row
                    # => increase data line index to add another data line to current dataset
                    # (relevant for player search results for example)
                    data_line_index += 1

                current_line_type = LineType.DATA
                datasets[-1].data.append(line[2:])
            elif line[:2] == '$\t':
                # Line starts with end marker => and stop parsing
                break
            elif current_line_type == LineType.HEADERS:
                # Line has no marker and last marker indicated a header line
                # => append line to header of current dataset
                datasets[-1].keys += line
            else:
                # Line has no marker and last marker indicated a header line
                # => append line to data of current dataset
                datasets[-1].data[data_line_index] += line

        return datasets

    @staticmethod
    def build_dict_from_datasets(datasets: List[Dataset], targets: List[ParseTarget]) -> dict:
        data = {}
        for index, dataset in enumerate(datasets):
            target = targets[index] if index < len(targets) else None
            if target is None:
                # Ever reaching this means we did not receive enough targets to handle all datasets,
                # which would usually be an implementation error
                raise Error('No parse target for aspx response dataset')

            # Split keys/headers into list
            keys = dataset.keys.split('\t')
            # Split each data line into a list
            data_lines = [line.split('\t') for line in dataset.data]

            if target.to_root:
                # Add dataset to dict root
                data.update({key: data_lines[0][index] for (index, key) in enumerate(keys)})
            elif len(data_lines) == 1 and not target.as_list:
                # Only a single line of data, add as properties under key (child object)
                # exception: player search returning only a single player (in that case, force return an array)
                data[target.to_key] = {
                    key: data_lines[0][index] if index < len(data_lines[0]) else ''
                    for (index, key) in enumerate(keys)
                }
            elif target.as_list:
                # Multiple lines of data, create list of dicts
                data[target.to_key] = [
                    {keys[index]: value for (index, value) in enumerate(data_line)}
                    for (index, data_line) in enumerate(data_lines)
                ]

        return data



from dataclasses import dataclass
from typing import Dict, Union, Optional


@dataclass
class AttributeSchema:
    type: type
    is_numeric: bool = False
    is_floaty: bool = False
    is_ratio: bool = False
    children: Optional[Dict[str, Union[dict, 'AttributeSchema']]] = None


def is_valid_dict(data: dict, schema: Dict[str, Union[dict, AttributeSchema]]) -> bool:
    return all(is_valid_attribute(data.get(key), attribute_schema) for (key, attribute_schema) in schema.items())


def is_valid_attribute(attribute: Union[str, dict, list], schema: Dict[str, Union[dict, AttributeSchema]]) -> bool:
    if isinstance(schema, AttributeSchema):
        if isinstance(attribute, str) and schema.type == str and schema.is_numeric:
            return is_numeric(attribute)
        if isinstance(attribute, str) and schema.type == str and schema.is_floaty:
            return is_floaty(attribute)
        if isinstance(attribute, str) and schema.type == str and schema.is_ratio:
            return is_ratio(attribute)
        if isinstance(attribute, list) and schema.type == list:
            return all(is_valid_attribute(child, schema.children) for child in attribute)
        else:
            return isinstance(attribute, schema.type)
    elif isinstance(attribute, dict) and isinstance(schema, dict):
        return is_valid_dict(attribute, schema)
    else:
        return False


def is_numeric(value: str) -> bool:
    """
    Test whether a string is parseable to int
    (used instead of str.isnumeric(), since that cannot handle negative numbers)
    """
    try:
        int(value)
        return True
    except ValueError:
        return False


def is_floaty(value: str) -> bool:
    """
    Test whether a string is parseable to float
    """
    try:
        float(value)
        return True
    except ValueError:
        return False


def is_ratio(value: str) -> bool:
    """
    Test whether a string is ratio of two integers ("123:789", with "0" being accepted as the zero value)
    """
    elements = value.split(':', 1)
    return len(elements) == 2 and all(is_numeric(elem) for elem in elements) or value == '0'



class Error(Exception):
    pass


class ClientError(Error):
    pass


class InvalidResponseError(Error):
    pass


class NotFoundError(Error):
    pass


class InvalidParameterError(Error):
    pass



from typing import Dict, Optional, Union

from .schemas import GETLEADERBOARD_RESPONSE_SCHEMA, SEARCHFORPLAYERS_RESPONSE_SCHEMA, \
    GETPLAYERINFO_GENERAL_STATS_RESPONSE_SCHEMA, GETPLAYERINFO_MAP_STATS_RESPONSE_SCHEMA
from .types import StatsProvider, SearchMatchType, SearchSortOrder, PlayerSearchResult, \
    PlayerSearchResponse, LeaderboardType, ScoreLeaderboardId, WeaponLeaderboardId, VehicleLeaderboardId, \
    KitLeaderboardId, LeaderboardEntry, LeaderboardResponse, PlayerinfoKeySet
from ..client import AspxClient as BaseAspxClient
from ..exceptions import InvalidParameterError, InvalidResponseError, NotFoundError
from ..types import ProviderConfig, ParseTarget, ResponseValidationMode
from ..validation import is_valid_dict, is_numeric


class AspxClient(BaseAspxClient):
    provider: StatsProvider

    def __init__(
            self,
            provider: StatsProvider = StatsProvider.BF2HUB,
            timeout: float = 2.0,
            response_validation_mode: ResponseValidationMode = ResponseValidationMode.LAX
    ):
        provider_config = AspxClient.get_provider_config(provider)
        super().__init__(provider_config.base_uri, provider_config.default_headers, timeout, response_validation_mode)
        self.provider = provider

    def searchforplayers(
            self,
            nick: str,
            where: SearchMatchType = SearchMatchType.EQUALS,
            sort: SearchSortOrder = SearchSortOrder.ASCENDING
    ) -> PlayerSearchResponse:
        parsed = self.searchforplayers_raw(nick, where, sort)

        # We can safely access keys and parse integers directly here,
        # since we already validated all used are present and all relevant strings are numeric
        return PlayerSearchResponse(
            asof=int(parsed['asof']),
            results=[
                PlayerSearchResult(
                    n=int(result['n']),
                    nick=result['nick'],
                    pid=int(result['pid']),
                    score=int(result['score'])
                ) for result in parsed['results']
            ]
        )

    def searchforplayers_raw(
            self,
            nick: str,
            where: SearchMatchType = SearchMatchType.EQUALS,
            sort: SearchSortOrder = SearchSortOrder.ASCENDING
    ) -> dict:
        raw_data = self.get_aspx_data('searchforplayers.aspx', {
            'nick': nick,
            'where': where,
            'sort': sort
        })

        valid_response, _ = self.is_valid_aspx_response(raw_data, self.response_validation_mode)
        if not valid_response:
            raise InvalidResponseError(f'{self.provider} returned an invalid searchforplayers response')

        parsed = self.parse_aspx_response(raw_data, [
            ParseTarget(to_root=True),
            ParseTarget('results', as_list=True)
        ])

        valid_data = self.is_valid_searchforplayers_response_data(parsed)
        if not valid_data:
            raise InvalidResponseError(f'{self.provider} returned invalid searchforplayers response data')

        return parsed

    @staticmethod
    def is_valid_searchforplayers_response_data(parsed: dict) -> bool:
        return is_valid_dict(parsed, SEARCHFORPLAYERS_RESPONSE_SCHEMA)

    def getleaderboard(
            self,
            leaderboard_type: LeaderboardType = LeaderboardType.SCORE,
            leaderboard_id: Union[
                ScoreLeaderboardId,
                WeaponLeaderboardId,
                VehicleLeaderboardId,
                KitLeaderboardId
            ] = ScoreLeaderboardId.OVERALL,
            pos: int = 1,
            before: int = 0,
            after: int = 19,
            pid: Optional[int] = None
    ) -> LeaderboardResponse:
        parsed = self.getleaderboard_raw(leaderboard_type, leaderboard_id, pos, before, after, pid)

        return LeaderboardResponse(
            size=int(parsed['size']),
            asof=int(parsed['asof']),
            entries=[
                LeaderboardEntry(
                    n=int(entry['n']),
                    pid=int(entry['pid']),
                    nick=entry['nick'],
                    rank=int(entry['playerrank']),
                    country_code=entry['countrycode']
                ) for entry in parsed['entries']
            ]
        )

    def getleaderboard_raw(
            self,
            leaderboard_type: LeaderboardType = LeaderboardType.SCORE,
            leaderboard_id: Union[
                ScoreLeaderboardId,
                WeaponLeaderboardId,
                VehicleLeaderboardId,
                KitLeaderboardId
            ] = ScoreLeaderboardId.OVERALL,
            pos: int = 1,
            before: int = 0,
            after: int = 19,
            pid: Optional[int] = None
    ) -> dict:
        # TODO Validate type and id combinations
        raw_data = self.get_aspx_data('getleaderboard.aspx', {
            'type': leaderboard_type,
            'id': leaderboard_id,
            'pos': pos,
            'before': before,
            'after': after,
            'pid': pid
        })

        valid_response, _ = self.is_valid_aspx_response(raw_data, self.response_validation_mode)
        if not valid_response:
            raise InvalidResponseError(f'{self.provider} returned an invalid getleaderboard response')

        parsed = self.parse_aspx_response(raw_data, [
            ParseTarget(to_root=True),
            ParseTarget('entries', as_list=True)
        ])

        valid_data = self.is_valid_getleaderboard_response_data(parsed)
        if not valid_data:
            raise InvalidResponseError(f'{self.provider} returned invalid getleaderboard response data')

        return parsed

    @staticmethod
    def is_valid_getleaderboard_response_data(parsed: dict) -> bool:
        # TODO: Add per-leaderboard validation with respective attributes
        return is_valid_dict(parsed, GETLEADERBOARD_RESPONSE_SCHEMA)

    def getplayerinfo_raw(
            self,
            pid: int,
            key_set: PlayerinfoKeySet = PlayerinfoKeySet.GENERAL_STATS
    ) -> dict:
        raw_data = self.get_aspx_data('getplayerinfo.aspx', {
            'pid': pid,
            'info': key_set
        })

        valid_response, not_found = self.is_valid_aspx_response(raw_data, self.response_validation_mode)
        if not valid_response and not_found:
            raise NotFoundError(f'No such player on {self.provider}')
        elif not valid_response:
            raise InvalidResponseError(f'{self.provider} returned an invalid getplayerinfo response')

        parsed = self.parse_aspx_response(raw_data, [
            ParseTarget(to_root=True),
            ParseTarget('player')
        ])

        parsed = self.fix_getplayerinfo_zero_values(parsed)

        valid_data = self.is_valid_getplayerinfo_response_data(key_set, parsed)
        if not valid_data:
            raise InvalidResponseError(f'{self.provider} returned invalid getplayerinfo response data')

        return parsed

    @staticmethod
    def fix_getplayerinfo_zero_values(parsed: dict) -> dict:
        # Can't fix any player attributes if the key is missing/of wrong type
        if not isinstance(parsed.get('player'), dict):
            return parsed

        """
        If a player has no kills/deaths, the PlayBF2 backend returns
        a whitespace instead of a zero integer value for:
        tvcr (top victim pid)
        topr (top opponent pid)
        mvrs (top victim rank)
        vmrs (top opponent rank)
        BF2Hub handles it better in most cases, but also has players with an empty string mvrs/vmrs or even more
        interesting values such as "NOT VAILABLE" for tvcr (pid 10226681 asof 1617839795)
        => replace any invalid values with 0 (but don't add it if the key is missing)
        """
        for key in ['tvcr', 'topr', 'mvrs', 'vmrs']:
            value = parsed['player'].get(key)
            if value is not None and not is_numeric(value):
                parsed['player'][key] = '0'

        return parsed

    @staticmethod
    def is_valid_getplayerinfo_response_data(key_set: PlayerinfoKeySet, parsed: dict) -> bool:
        if key_set == PlayerinfoKeySet.GENERAL_STATS:
            return is_valid_dict(parsed, GETPLAYERINFO_GENERAL_STATS_RESPONSE_SCHEMA)
        else:
            return is_valid_dict(parsed, GETPLAYERINFO_MAP_STATS_RESPONSE_SCHEMA)

    @staticmethod
    def get_provider_config(provider: StatsProvider = StatsProvider.BF2HUB) -> ProviderConfig:
        provider_configs: Dict[StatsProvider, ProviderConfig] = {
            StatsProvider.BF2HUB: ProviderConfig(
                base_uri='http://official.ranking.bf2hub.com/ASP/',
                default_headers={
                    'Host': 'BF2web.gamespy.com',
                    'User-Agent': 'GameSpyHTTP/1.0'
                }
            ),
            StatsProvider.PLAYBF2: ProviderConfig(
                base_uri='http://bf2web.playbf2.ru/ASP/'
            )
        }

        config = provider_configs.get(provider, None)
        if config is None:
            raise InvalidParameterError(f'No provider config for given provider "{provider}"')

        return config



from dataclasses import dataclass
from enum import Enum
from typing import List


class StatsProvider(str, Enum):
    BF2HUB = 'bf2hub'
    PLAYBF2 = 'playbf2'


class SearchMatchType(str, Enum):
    CONTAINS = 'a'
    BEGINS_WITH = 'b'
    ENDS_WITH = 'e'
    EQUALS = 'x'


class SearchSortOrder(str, Enum):
    ASCENDING = 'a'
    DESCENDING = 'r'


@dataclass
class PlayerSearchResult:
    n: int
    pid: int
    nick: str
    score: int

    def __iter__(self):
        yield 'n', self.n
        yield 'pid', self.pid
        yield 'nick', self.nick
        yield 'score', self.score


@dataclass
class PlayerSearchResponse:
    asof: int
    results: List[PlayerSearchResult]

    def __iter__(self):
        yield 'asof', self.asof
        yield 'results', [dict(result) for result in self.results]


class LeaderboardType(str, Enum):
    SCORE = 'score'
    WEAPON = 'weapon'
    VEHICLE = 'vehicle'
    KIT = 'kit'


class ScoreLeaderboardId(str, Enum):
    # Score leaderboards use names as ids
    OVERALL = 'overall'
    RISING_STAR = 'risingstar'
    COMMANDER = 'commander'
    TEAM = 'team'
    COMBAT = 'combat'


class WeaponLeaderboardId(int, Enum):
    ASSAULT_RIFLE = 0
    GRENADE_LAUNCHER = 1
    CARBINE = 2
    LIGHT_MACHINE_GUN = 3
    SNIPER_RIFLE = 4
    PISTOL = 5
    ANTI_TANK = 6
    SUB_MACHINE_GUN = 7
    SHOTGUN = 8
    KNIFE = 9
    DEFIBRILLATOR = 10
    EXPLOSIVES = 11
    GRENADE = 12


class VehicleLeaderboardId(int, Enum):
    ARMOR = 0
    JET = 1
    ANTI_AIR = 2
    HELICOPTER = 3
    TRANSPORT = 4
    GROUND_DEFENSE = 5


class KitLeaderboardId(int, Enum):
    ANTI_TANK = 0
    ASSAULT = 1
    ENGINEER = 2
    MEDIC = 3
    SPEC_OPS = 4
    SUPPORT = 5
    SNIPER = 6


@dataclass
class LeaderboardEntry:
    n: int
    pid: int
    nick: str
    rank: int
    country_code: str

    def __iter__(self):
        yield 'n', self.n
        yield 'pid', self.pid
        yield 'nick', self.nick
        yield 'rank', self.rank
        yield 'country_code', self.country_code


@dataclass
class LeaderboardResponse:
    size: int
    asof: int
    entries: List[LeaderboardEntry]

    def __iter__(self):
        yield 'size', self.size
        yield 'asof', self.asof
        yield 'entries', [dict(entry) for entry in self.entries]


class PlayerinfoKeySet(str, Enum):
    GENERAL_STATS = 'per*,cmb*,twsc,cpcp,cacp,dfcp,kila,heal,rviv,rsup,rpar,tgte,dkas,dsab,cdsc,rank,cmsc,kick,kill,' \
                    'deth,suic,ospm,klpm,klpr,dtpr,bksk,wdsk,bbrs,tcdr,ban,dtpm,lbtl,osaa,vrk,tsql,tsqm,tlwf,mvks,' \
                    'vmks,mvn*,vmr*,fkit,fmap,fveh,fwea,wtm-,wkl-,wdt-,wac-,wkd-,vtm-,vkl-,vdt-,vkd-,vkr-,atm-,awn-,' \
                    'alo-,abr-,ktm-,kkl-,kdt-,kkd-'
    MAP_STATS = 'mtm-,mwn-,mls-'



from .exceptions import Error, ClientError, InvalidResponseError, NotFoundError, InvalidParameterError

"""
aspxstats.
Python library for retrieving stats of Battlefield 2 and Battlefield 2142 players.
"""

__version__ = '0.0.1'
__author__ = 'cetteup'
__credits__ = 'wilson212'
__all__ = ['bf2', 'Error', 'ClientError', 'InvalidResponseError', 'NotFoundError', 'InvalidParameterError']



from .client import AspxClient
from .fetch import searchforplayers, searchforplayers_raw, getleaderboard, getleaderboard_raw, getplayerinfo_raw
from .types import StatsProvider, SearchMatchType, SearchSortOrder, LeaderboardType, ScoreLeaderboardId, \
    WeaponLeaderboardId, VehicleLeaderboardId, KitLeaderboardId, PlayerinfoKeySet

__all__ = [
    'AspxClient',
    'searchforplayers',
    'searchforplayers_raw',
    'getleaderboard',
    'getleaderboard_raw',
    'getplayerinfo_raw',
    'StatsProvider',
    'SearchMatchType',
    'SearchSortOrder',
    'LeaderboardType',
    'ScoreLeaderboardId',
    'WeaponLeaderboardId',
    'VehicleLeaderboardId',
    'KitLeaderboardId',
    'PlayerinfoKeySet'
]
